package tb3d;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * <p>Title: TreeBuilder3D</p>
 * <p>Description: Interactive 3D viewer</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: GSC BC Cancer Agency</p>
 *
 * @author Peter Ruzanov
 * @version 1.0
 */

/*GPL
This file is part of TreeBuilder3D

TreeBuilder3D is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

TreeBuilder3D is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeBuilder3D; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

class tb3d_stat{
public static final int CHI  = 1;
public static final int CORR = 2;
public static final int EUC  = 3;
public static final int PRSUC= 4;
public static final int MAN  = 5;
/**
 * Calculation of distance between two datasets: return number of differently expressed genes.
 * @param M Special class holding the data for a pair of SAGE (or Microarray) libs
 * @param Metric the id of metric to be used for distance matrix generation
 * @return dist - double value for the distance between two libraries
 */

public static double getDist(tb3d_controls.Mix M, int Metric){
    double dist = 0;
    Enumeration m = M.genes.elements();
    //If we don't have totals, assume that normalization to 100000 has been done
    if(M.Atotal <= 0 && M.Btotal <= 0){
	 M.Atotal = 100000;
	 M.Btotal = 100000;
    }

switch (Metric){
case CHI:
//******************************************************
//---------------QUANTITATIVE---------------------------
//******************************************************
	int sim = 0;
while(m.hasMoreElements()){
	tb3d_controls.Gene current = (tb3d_controls.Gene)m.nextElement();

	double chi = ((current.count/M.Atotal-current.alt/M.Btotal)*(current.count/M.Atotal-current.alt/M.Btotal))/
		((current.count/M.Atotal+current.alt/M.Btotal)/2*(1-(current.count/M.Atotal+current.alt/M.Btotal)/2)*
		(1.0/M.Atotal+1.0/M.Btotal));

	if(chi > 3.841)//The constant signifies the threshold p=0.05
	//  if(chi > 6.635)
	    dist ++;
	  else
	    sim++;
	}
if(sim == 0)
	  sim = 1;
dist/=sim;
break;
case CORR:
	double top1 = 0;
	double top2 = 0;
	double top3 = 0;
	double sum1 = 0;
	double sum2 = 0;
//********************************************************
//----------------PEARSON CORRELATION SQUARED-------------
//********************************************************
while(m.hasMoreElements()){
	  tb3d_controls.Gene current = (tb3d_controls.Gene)m.nextElement();

	  top1   += current.count/M.Atotal*current.alt/M.Btotal;
	  top2   += current.count/M.Atotal;
	  top3   += current.alt/M.Btotal;

	  sum1 += (current.count/M.Atotal)*(current.count/M.Atotal);
	  sum2 += (current.alt/M.Btotal)*(current.alt/M.Btotal);
}
	  dist = Math.sqrt((sum1-sum1/M.genes.size())*(sum2-sum2/M.genes.size()))/(top1-top2*top3/M.genes.size());//--------SQUARED INVERTED PEARSON COEFF--------------
	  dist *= dist;
break;

case PRSUC:
	  double topp1 = 0;
	  double topp2 = 0;
	  double topp3 = 0;
	  double summ1 = 0;
	  double summ2 = 0;
//********************************************************
//-------UNCENTERED PEARSON CORRELATION SQUARED-----------
//********************************************************
while(m.hasMoreElements()){
	  tb3d_controls.Gene current = (tb3d_controls.Gene)m.nextElement();
	  topp1   += current.count/M.Atotal*current.alt/M.Btotal;
	  topp2   += current.count/M.Atotal;
	  topp3   += current.alt/M.Btotal;

	  summ1 += (current.count/M.Atotal)*(current.count/M.Atotal);
	  summ2 += (current.alt/M.Btotal)*(current.alt/M.Btotal);

	}
	dist = Math.sqrt(summ1*summ2)/(topp1-topp2*topp3/M.genes.size());//--------INVERTED PEARSON COEFF--------------
	dist *= dist;
	break;

case EUC:
//**********************************************************
//----------------EUCLIDEAN---------------------------------
//**********************************************************
while(m.hasMoreElements()){
	  tb3d_controls.Gene current = (tb3d_controls.Gene)m.nextElement();
dist += Math.sqrt((current.count/M.Atotal-current.alt/M.Btotal)*(current.count/M.Atotal-current.alt/M.Btotal));
}
break;

case MAN:
//**********************************************************
//----------------MANHATTAN---------------------------------
//**********************************************************
while(m.hasMoreElements()){
    tb3d_controls.Gene current = (tb3d_controls.Gene)m.nextElement();
    dist += Math.abs(current.count/M.Atotal-current.alt/M.Btotal);
    }
break;
default:
break;
}
return dist;
}
}
